@@ -1,11 +1,12 @@
-import json
 import time
-from typing import List
+from typing import List, Union
 
 from worker.app.modules import Module
-
 from worker.app.modules.time_activity_module import TimeActivityModule
 from worker.data.operations import gather_data_for_period
+from worker.event import Event, EventType
+from worker.event.scheduled import ScheduledEvent
+from worker.event.stream import StreamEvent
 from worker.framework import constants
 from worker.mixins.logging import LoggingMixin
 from worker.mixins.rollbar import RollbarMixin
@@ -15,6 +16,9 @@ from worker import exceptions
 
 
 class App(StateMixin, LoggingMixin, RollbarMixin):
+    """
+    The apps is designed to receive the events of a single asset
+    """
     app_state_fields = {
         'asset_id': int,
         'last_processed_timestamp': int
@@ -30,31 +34,33 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
             # Add to kwargs if defined in constants, else will get from environment
             kwargs["state_storage_type"] = state_storage_type
 
+        self.event_type: EventType = None
+
         self.asset_id = None
-        self.event = None
+        self.event_records = None
 
         super().__init__(*args, **kwargs)
 
-    def load(self, event: str):
+    def load(self, event_type: EventType, event: Event):
         """
         :param event: a scheduler event or wits stream
         :return:
         """
-        event_type = self.get_event_type()
-        valid_stream_collections = self.get_valid_stream_collections()
-        event = self.clean_event(event)
+        self.event_type = event_type
 
         max_lookback_seconds = self.get_max_lookback_seconds()
-        event = self.format_event(event_type, event)
-        self.asset_id = self.determine_asset_id(event)
+        self.asset_id = event.asset_id
 
         self.state = self.load_state()
 
-        event = self.load_event(event_type, event, max_lookback_seconds)
-        self.event = self.filter_event_for_collections(event_type, valid_stream_collections, event)
-        self.log_event(self.event, max_lookback_seconds)
+        event = self.load_event(event, max_lookback_seconds)
 
-    def log_event(self, event, max_lookback_seconds):
+        valid_stream_collections = self.get_valid_stream_collections()
+        self.event_records = self.filter_event_for_collections(self.event_type, valid_stream_collections, event)
+
+        self.log_event(self.event_records, max_lookback_seconds)
+
+    def log_event(self, event: List[dict], max_lookback_seconds: int):
         self.debug(self.asset_id, "WITS input to {0} -> {1}".format(self.app_name, event))
 
         if not event:
@@ -72,15 +78,6 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
             )
         )
 
-    @staticmethod
-    def get_event_type():
-        event_type = constants.get('global.event-type', "")
-
-        if event_type not in ("scheduler", "wits_stream"):
-            raise Exception('event_type specified incorrectly: {}'.format(event_type))
-
-        return event_type
-
     @staticmethod
     def get_valid_stream_collections():
         valid_collections = constants.get('global.valid-stream-collections', [])
@@ -91,19 +88,6 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
             valid_collections = [valid_collections]
         return valid_collections
 
-    @staticmethod
-    def clean_event(event):
-        if not event:
-            raise Exception("Empty event")
-
-        try:
-            if isinstance(event, (str, bytes, bytearray)):
-                event = json.loads(event)
-        except ValueError:
-            raise Exception("Invalid event JSON")
-
-        return event
-
     def get_max_lookback_seconds(self):
         """
         For each module (mostly in time-base modules), the time of processing does not
@@ -120,17 +104,17 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
 
         return maximum_lookback
 
-    def load_event(self, event_type, event, max_lookback_seconds):
-        if event_type == 'scheduler':
+    def load_event(self, event: Event, max_lookback_seconds: int) -> List[dict]:
+        if self.event_type == EventType.SCHEDULER:
             return self.load_scheduler_event(self.asset_id, event, max_lookback_seconds)
 
-        if event_type == 'wits_stream':
+        if self.event_type == EventType.WITS_STREAM:
             return self.load_wits_stream_event(self.asset_id, event, max_lookback_seconds)
 
         return None
 
     @staticmethod
-    def filter_event_for_collections(event_type: str, valid_stream_collections: list, event: list):
+    def filter_event_for_collections(event_type: EventType, valid_stream_collections: list, event: list) -> list:
         """
         This function filters the incoming event based on a list of valid collections
 
@@ -141,7 +125,7 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
         """
 
         # If event type is scheduler or event_collections is empty, return the entire event
-        if event_type == 'scheduler' or not valid_stream_collections:
+        if event_type == EventType.SCHEDULER or not valid_stream_collections:
             return event
 
         # Filtering the event based on valid collections
@@ -170,17 +154,17 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
 
         return filtered_events
 
-    def load_scheduler_event(self, asset_id, event, max_lookback_seconds):
+    def load_scheduler_event(self, asset_id: int, event: ScheduledEvent, max_lookback_seconds: int) -> List[dict]:
         """
         To load a scheduler event and get the wits stream data
         :param asset_id: The asset to load
-        :param event: The cleaned event from Kafka scheduler stream
+        :param event: A cleaned event
         :param max_lookback_seconds: Maximum amount of time to look back prior to the scheduler event to cover gaps
         :return: list of WITS data between the last processed timestamp and the final event item timestamp
         """
 
-        start_timestamp = self.state.get('last_processed_timestamp', event[0].get('timestamp') - 1)
-        end_timestamp = event[-1].get('timestamp')
+        start_timestamp = self.state.get('last_processed_timestamp', event[0].start_time - 1)
+        end_timestamp = event[-1].start_time
 
         # the event is converted from scheduler to wits stream
         scheduler_event = gather_data_for_period(
@@ -191,24 +175,24 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
 
         return scheduler_event
 
-    def load_wits_stream_event(self, asset_id, event, max_lookback_seconds):
+    def load_wits_stream_event(self, asset_id: int, event: StreamEvent, max_lookback_seconds: int) -> List[dict]:
         """
         To load a wits stream event and get more data if necessary
         :param asset_id: The asset to load
-        :param event: The cleaned event from Kafka WITS stream
+        :param event: A cleaned event
         :param max_lookback_seconds: Maximum amount of time to look back prior to WITS data to cover gaps
         :return: list of WITS data between the first event timestamp and the first timestamp
         """
+        records = event.records
 
-        if event and max_lookback_seconds:
-            # First filtering original event for duplicates
-            event = self.filter_event_for_duplicates(event)
+        # First filtering original event for duplicates
+        records = self.filter_event_for_duplicates(records)
 
-            first_timestamp = event[0].get('timestamp')
-
-            if not first_timestamp:
-                return event
+        first_timestamp = records[0].get('timestamp')
+        if not first_timestamp:
+            return records
 
+        if max_lookback_seconds:
             # Subtract one from the timestamp so that we don't reselect the final data item that was sent in the event
             end_timestamp = first_timestamp - 1
 
@@ -216,57 +200,12 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
                 asset_id=asset_id,
                 start=first_timestamp - max_lookback_seconds,
                 end=end_timestamp,
-                limit=constants.get('global.query-limit'))
+                limit=constants.get('global.query-limit')
+            )
 
-            event = previous_events + event
+            records = previous_events + records
 
-        return event
-
-    @staticmethod
-    def format_event(event_type: str, event: list) -> list:
-        """
-        validate the wits_stream event, flatten and organize the data into a desired format
-        :param event_type: type of event
-        :param event: a wits_stream event
-        :return: a list of wits records
-        """
-
-        first_event = event[0]
-        if isinstance(first_event, list):
-            # This is possible for both scheduler and old wits stream event types
-            try:
-                # Because the events are structured into 'list of lists', using the following code
-                # converts them into a flat single list. This is done for better data processing in
-                # the proceeding steps.
-                event = [item for sublist in event for item in sublist]
-            except ValueError:
-                raise Exception("Records are not valid JSON")
-
-            if not isinstance(event, list):
-                raise Exception("Records is not an array")
-
-            if not event:
-                raise Exception("Records is empty.")
-
-            if event_type == 'scheduler':
-                event = [
-                    {
-                        "asset_id": item["asset_id"],
-                        "timestamp": int(item['schedule_start'] / 1000)
-                    }
-                    for item in event
-                ]
-            return event
-
-        elif isinstance(first_event, dict):
-            # This is possible for the new wits stream event type
-            records = []
-            for each in event:
-                records = records + each.get("records", [])
-            return records
-
-        else:
-            raise TypeError("Event is neither a scheduler, old nor new kafka consumer")
+        return records
 
     @staticmethod
     def determine_asset_id(event: list) -> int:
@@ -275,7 +214,7 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
         except Exception:
             raise Exception('Event does not contain asset_id: {}'.format(event))
 
-    def load_state(self, state_key: [str, None] = None, auto_migrate: bool = False, raise_warnings: bool = False) -> dict:
+    def load_state(self, state_key: Union[str, None] = None, auto_migrate: bool = False, raise_warnings: bool = False) -> dict:
         previous_state = super().load_state(state_key=state_key, auto_migrate=auto_migrate, raise_warnings=raise_warnings)
 
         state = State(self.app_state_fields, previous_state)
@@ -285,7 +224,7 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
 
         return state
 
-    def save_state(self, state_key: [str, None] = None, timestamp: [int, None] = None, raise_warnings: bool = True) -> None:
+    def save_state(self, state_key: Union[str, None] = None, timestamp: Union[int, None] = None, raise_warnings: bool = True) -> None:
         if not timestamp:
             timestamp = self.state.get('last_processed_timestamp', 0)
 
@@ -298,7 +237,7 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
         return [module for module in self.get_modules() if module.enabled]
 
     def run_modules(self):
-        if not self.event:
+        if not self.event_records:
             return
 
         for module_type in self.get_active_modules():
@@ -308,11 +247,11 @@ class App(StateMixin, LoggingMixin, RollbarMixin):
                 raise exceptions.Misconfigured("Module {0} not able to initialize for asset_id {1}".format(module_type, self.asset_id))
 
             try:
-                module.run(self.event)
+                module.run(self.event_records)
             except Exception:
                 message = f"Error in module {module_type.module_key}"
                 self.track_error(message=message)
                 raise
 
-        last_processed_timestamp = self.event[-1].get('timestamp')
+        last_processed_timestamp = self.event_records[-1].get('timestamp')
         self.state['last_processed_timestamp'] = last_processed_timestamp
