@@ -17,6 +17,7 @@
 #     })
 
 
+from worker.data.api import API
 from worker.data.operations import get_data_by_path
 
 parameters = {
@@ -47,3 +48,7 @@ def update(additional_parameters):
     parameters.update(additional_parameters)
     if globals:
         parameters['global'] = globals
+
+
+# a default api that can be used in any part of the code
+api = API()
