@@ -23,6 +23,7 @@ class PipeType(Enum):
     MWD = "Measurement While Drilling"
     RSS = "Rotary Steerable System"
     BIT = "Bit"
+    UNDERREAMER = "Under-reamer"
 
     @staticmethod
     def determine_type(family: str):
@@ -36,7 +37,8 @@ class PipeType(Enum):
             'pdm': PipeType.PDM,
             'mwd': PipeType.MWD,
             'rss': PipeType.RSS,
-            'bit': PipeType.BIT
+            'bit': PipeType.BIT,
+            'ur': PipeType.UNDERREAMER
         }
         _type = mapping.get(family)
         if not _type:
